# coding: utf-8

"""
    Fabric Orchestrator API

    This is Fabric Orchestrator API  # noqa: E501

    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Status404NotFound(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'errors': 'list[Status404NotFoundErrors]',
        'type': 'str',
        'size': 'int',
        'status': 'int'
    }

    attribute_map = {
        'errors': 'errors',
        'type': 'type',
        'size': 'size',
        'status': 'status'
    }

    def __init__(self, errors=None, type='error', size=1, status=404):  # noqa: E501
        """Status404NotFound - a model defined in Swagger"""  # noqa: E501
        self._errors = None
        self._type = None
        self._size = None
        self._status = None
        self.discriminator = None
        if errors is not None:
            self.errors = errors
        if type is not None:
            self.type = type
        if size is not None:
            self.size = size
        if status is not None:
            self.status = status

    @property
    def errors(self):
        """Gets the errors of this Status404NotFound.  # noqa: E501


        :return: The errors of this Status404NotFound.  # noqa: E501
        :rtype: list[Status404NotFoundErrors]
        """
        return self._errors

    @errors.setter
    def errors(self, errors):
        """Sets the errors of this Status404NotFound.


        :param errors: The errors of this Status404NotFound.  # noqa: E501
        :type: list[Status404NotFoundErrors]
        """

        self._errors = errors

    @property
    def type(self):
        """Gets the type of this Status404NotFound.  # noqa: E501


        :return: The type of this Status404NotFound.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Status404NotFound.


        :param type: The type of this Status404NotFound.  # noqa: E501
        :type: str
        """

        self._type = type

    @property
    def size(self):
        """Gets the size of this Status404NotFound.  # noqa: E501


        :return: The size of this Status404NotFound.  # noqa: E501
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """Sets the size of this Status404NotFound.


        :param size: The size of this Status404NotFound.  # noqa: E501
        :type: int
        """

        self._size = size

    @property
    def status(self):
        """Gets the status of this Status404NotFound.  # noqa: E501


        :return: The status of this Status404NotFound.  # noqa: E501
        :rtype: int
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this Status404NotFound.


        :param status: The status of this Status404NotFound.  # noqa: E501
        :type: int
        """

        self._status = status

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Status404NotFound, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Status404NotFound):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Fabric Orchestrator API

    This is Fabric Orchestrator API  # noqa: E501

    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

from fabric_cf.orchestrator.swagger_client.models.slice import Slice  # noqa: E501
from fabric_cf.orchestrator.swagger_client.rest import ApiException


class TestSlice(unittest.TestCase):
    """Slice unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testSlice(self):
        """Test Slice"""
        # FIXME: construct object with mandatory attributes with example values
        # model = fabric_cf.orchestrator.swagger_client.models.slice.Slice()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Fabric Orchestrator API

    This is Fabric Orchestrator API  # noqa: E501

    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PoaPostData(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'vcpu_cpu_map': 'list[PoaPostDataVcpuCpuMap]',
        'node_set': 'list[str]',
        'keys': 'list[PoaPostDataKeys]'
    }

    attribute_map = {
        'vcpu_cpu_map': 'vcpu_cpu_map',
        'node_set': 'node_set',
        'keys': 'keys'
    }

    def __init__(self, vcpu_cpu_map=None, node_set=None, keys=None):  # noqa: E501
        """PoaPostData - a model defined in Swagger"""  # noqa: E501
        self._vcpu_cpu_map = None
        self._node_set = None
        self._keys = None
        self.discriminator = None
        if vcpu_cpu_map is not None:
            self.vcpu_cpu_map = vcpu_cpu_map
        if node_set is not None:
            self.node_set = node_set
        if keys is not None:
            self.keys = keys

    @property
    def vcpu_cpu_map(self):
        """Gets the vcpu_cpu_map of this PoaPostData.  # noqa: E501


        :return: The vcpu_cpu_map of this PoaPostData.  # noqa: E501
        :rtype: list[PoaPostDataVcpuCpuMap]
        """
        return self._vcpu_cpu_map

    @vcpu_cpu_map.setter
    def vcpu_cpu_map(self, vcpu_cpu_map):
        """Sets the vcpu_cpu_map of this PoaPostData.


        :param vcpu_cpu_map: The vcpu_cpu_map of this PoaPostData.  # noqa: E501
        :type: list[PoaPostDataVcpuCpuMap]
        """

        self._vcpu_cpu_map = vcpu_cpu_map

    @property
    def node_set(self):
        """Gets the node_set of this PoaPostData.  # noqa: E501


        :return: The node_set of this PoaPostData.  # noqa: E501
        :rtype: list[str]
        """
        return self._node_set

    @node_set.setter
    def node_set(self, node_set):
        """Sets the node_set of this PoaPostData.


        :param node_set: The node_set of this PoaPostData.  # noqa: E501
        :type: list[str]
        """

        self._node_set = node_set

    @property
    def keys(self):
        """Gets the keys of this PoaPostData.  # noqa: E501


        :return: The keys of this PoaPostData.  # noqa: E501
        :rtype: list[PoaPostDataKeys]
        """
        return self._keys

    @keys.setter
    def keys(self, keys):
        """Sets the keys of this PoaPostData.


        :param keys: The keys of this PoaPostData.  # noqa: E501
        :type: list[PoaPostDataKeys]
        """

        self._keys = keys

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PoaPostData, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PoaPostData):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Fabric Orchestrator API

    This is Fabric Orchestrator API  # noqa: E501

    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import swagger_client
from swagger_client.api.slivers_api import SliversApi  # noqa: E501
from swagger_client.rest import ApiException


class TestSliversApi(unittest.TestCase):
    """SliversApi unit test stubs"""

    def setUp(self):
        self.api = SliversApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_slivers_get(self):
        """Test case for slivers_get

        Retrieve a listing of user slivers  # noqa: E501
        """
        pass

    def test_slivers_sliver_id_get(self):
        """Test case for slivers_sliver_id_get

        slivers properties  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Fabric Orchestrator API

    This is Fabric Orchestrator API  # noqa: E501

    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import swagger_client
from swagger_client.api.poas_api import PoasApi  # noqa: E501
from swagger_client.rest import ApiException


class TestPoasApi(unittest.TestCase):
    """PoasApi unit test stubs"""

    def setUp(self):
        self.api = PoasApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_poas_create_sliver_id_post(self):
        """Test case for poas_create_sliver_id_post

        Perform an operational action on a sliver.  # noqa: E501
        """
        pass

    def test_poas_get(self):
        """Test case for poas_get

        Request get the status of the POAs.  # noqa: E501
        """
        pass

    def test_poas_poa_id_get(self):
        """Test case for poas_poa_id_get

        Perform an operational action on a sliver.  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Fabric Orchestrator API

    This is Fabric Orchestrator API  # noqa: E501

    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from fabric_cf.orchestrator.swagger_client.api_client import ApiClient


class PoasApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def poas_create_sliver_id_post(self, body, sliver_id, **kwargs):  # noqa: E501
        """Perform an operational action on a sliver.  # noqa: E501

        Request to perform an operation action on a sliver. Supported actions include - reboot a VM sliver, get cpu info, get numa info, pin vCPUs, pin memory to a numa node etc, add/remove ssh keys.     # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.poas_create_sliver_id_post(body, sliver_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param PoaPost body: Perform Operation Action (required)
        :param str sliver_id: Sliver identified by universally unique identifier (required)
        :return: Poa
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.poas_create_sliver_id_post_with_http_info(body, sliver_id, **kwargs)  # noqa: E501
        else:
            (data) = self.poas_create_sliver_id_post_with_http_info(body, sliver_id, **kwargs)  # noqa: E501
            return data

    def poas_create_sliver_id_post_with_http_info(self, body, sliver_id, **kwargs):  # noqa: E501
        """Perform an operational action on a sliver.  # noqa: E501

        Request to perform an operation action on a sliver. Supported actions include - reboot a VM sliver, get cpu info, get numa info, pin vCPUs, pin memory to a numa node etc, add/remove ssh keys.     # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.poas_create_sliver_id_post_with_http_info(body, sliver_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param PoaPost body: Perform Operation Action (required)
        :param str sliver_id: Sliver identified by universally unique identifier (required)
        :return: Poa
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'sliver_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method poas_create_sliver_id_post" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `poas_create_sliver_id_post`")  # noqa: E501
        # verify the required parameter 'sliver_id' is set
        if ('sliver_id' not in params or
                params['sliver_id'] is None):
            raise ValueError("Missing the required parameter `sliver_id` when calling `poas_create_sliver_id_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'sliver_id' in params:
            path_params['sliver_id'] = params['sliver_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['bearerAuth']  # noqa: E501

        return self.api_client.call_api(
            '/poas/create/{sliver_id}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Poa',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def poas_get(self, **kwargs):  # noqa: E501
        """Request get the status of the POAs.  # noqa: E501

        Request get the status of the POAs     # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.poas_get(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str sliver_id: Search for POAs for a sliver
        :param list[str] states: Search for POAs in the specified states
        :param int limit: maximum number of results to return per page (1 or more)
        :param int offset: number of items to skip before starting to collect the result set
        :return: Poa
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.poas_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.poas_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def poas_get_with_http_info(self, **kwargs):  # noqa: E501
        """Request get the status of the POAs.  # noqa: E501

        Request get the status of the POAs     # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.poas_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str sliver_id: Search for POAs for a sliver
        :param list[str] states: Search for POAs in the specified states
        :param int limit: maximum number of results to return per page (1 or more)
        :param int offset: number of items to skip before starting to collect the result set
        :return: Poa
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['sliver_id', 'states', 'limit', 'offset']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method poas_get" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'sliver_id' in params:
            query_params.append(('sliver_id', params['sliver_id']))  # noqa: E501
        if 'states' in params:
            query_params.append(('states', params['states']))  # noqa: E501
            collection_formats['states'] = 'multi'  # noqa: E501
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['bearerAuth']  # noqa: E501

        return self.api_client.call_api(
            '/poas/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Poa',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def poas_poa_id_get(self, poa_id, **kwargs):  # noqa: E501
        """Perform an operational action on a sliver.  # noqa: E501

        Request get the status of the POA identified by poa_id.     # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.poas_poa_id_get(poa_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str poa_id: Poa Id for the POA triggered (required)
        :return: Poa
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.poas_poa_id_get_with_http_info(poa_id, **kwargs)  # noqa: E501
        else:
            (data) = self.poas_poa_id_get_with_http_info(poa_id, **kwargs)  # noqa: E501
            return data

    def poas_poa_id_get_with_http_info(self, poa_id, **kwargs):  # noqa: E501
        """Perform an operational action on a sliver.  # noqa: E501

        Request get the status of the POA identified by poa_id.     # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.poas_poa_id_get_with_http_info(poa_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str poa_id: Poa Id for the POA triggered (required)
        :return: Poa
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['poa_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method poas_poa_id_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'poa_id' is set
        if ('poa_id' not in params or
                params['poa_id'] is None):
            raise ValueError("Missing the required parameter `poa_id` when calling `poas_poa_id_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'poa_id' in params:
            path_params['poa_id'] = params['poa_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['bearerAuth']  # noqa: E501

        return self.api_client.call_api(
            '/poas/{poa_id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Poa',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    Fabric Orchestrator API
    This is Fabric Orchestrator API  # noqa: E501
    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from fabric_cf.orchestrator.swagger_client.api_client import ApiClient


class ResourcesApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def portalresources_get(self, graph_format, **kwargs):  # noqa: E501
        """Retrieve a listing and description of available resources for portal  # noqa: E501

        Retrieve a listing and description of available resources for portal  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.portalresources_get(graph_format, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str graph_format: graph format (required)
        :param int level: Level of details
        :param bool force_refresh: Force to retrieve current available resource information.
        :param str start_date: starting date to check availability from
        :param str end_date: end date to check availability until
        :param str includes: comma separated lists of sites to include
        :param str excludes: comma separated lists of sites to exclude
        :return: Resources
                 If the method is called asynchronously,
                 returns the request thread.
        """
        # Remove all arguments with None value from kwargs
        kwargs = {key: value for key, value in kwargs.items() if value is not None}
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.portalresources_get_with_http_info(graph_format, **kwargs)  # noqa: E501
        else:
            (data) = self.portalresources_get_with_http_info(graph_format, **kwargs)  # noqa: E501
            return data

    def portalresources_get_with_http_info(self, graph_format, **kwargs):  # noqa: E501
        """Retrieve a listing and description of available resources for portal  # noqa: E501

        Retrieve a listing and description of available resources for portal  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.portalresources_get_with_http_info(graph_format, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str graph_format: graph format (required)
        :param int level: Level of details
        :param bool force_refresh: Force to retrieve current available resource information.
        :param str start_date: starting date to check availability from
        :param str end_date: end date to check availability until
        :param str includes: comma separated lists of sites to include
        :param str excludes: comma separated lists of sites to exclude
        :return: Resources
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['graph_format', 'level', 'force_refresh', 'start_date', 'end_date', 'includes', 'excludes']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method portalresources_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'graph_format' is set
        if ('graph_format' not in params or
                params['graph_format'] is None):
            raise ValueError("Missing the required parameter `graph_format` when calling `portalresources_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'graph_format' in params:
            query_params.append(('graph_format', params['graph_format']))  # noqa: E501
        if 'level' in params:
            query_params.append(('level', params['level']))  # noqa: E501
        if 'force_refresh' in params:
            query_params.append(('force_refresh', params['force_refresh']))  # noqa: E501
        if 'start_date' in params:
            query_params.append(('start_date', params['start_date']))  # noqa: E501
        if 'end_date' in params:
            query_params.append(('end_date', params['end_date']))  # noqa: E501
        if 'includes' in params:
            query_params.append(('includes', params['includes']))  # noqa: E501
        if 'excludes' in params:
            query_params.append(('excludes', params['excludes']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/portalresources', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Resources',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def resources_get(self, level, force_refresh, **kwargs):  # noqa: E501
        """Retrieve a listing and description of available resources. By default, a cached available resource information is returned. User can force to request the current available resources.  # noqa: E501

        Retrieve a listing and description of available resources. By default, a cached available resource information is returned. User can force to request the current available resources.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.resources_get(level, force_refresh, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int level: Level of details (required)
        :param bool force_refresh: Force to retrieve current available resource information. (required)
        :param str start_date: starting date to check availability from
        :param str end_date: end date to check availability until
        :param str includes: comma separated lists of sites to include
        :param str excludes: comma separated lists of sites to exclude
        :return: Resources
                 If the method is called asynchronously,
                 returns the request thread.
        """
        # Remove all arguments with None value from kwargs
        kwargs = {key: value for key, value in kwargs.items() if value is not None}
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.resources_get_with_http_info(level, force_refresh, **kwargs)  # noqa: E501
        else:
            (data) = self.resources_get_with_http_info(level, force_refresh, **kwargs)  # noqa: E501
            return data

    def resources_get_with_http_info(self, level, force_refresh, **kwargs):  # noqa: E501
        """Retrieve a listing and description of available resources. By default, a cached available resource information is returned. User can force to request the current available resources.  # noqa: E501
        Retrieve a listing and description of available resources. By default, a cached available resource information is returned. User can force to request the current available resources.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.resources_get_with_http_info(level, force_refresh, async_req=True)
        >>> result = thread.get()
        :param async_req bool
        :param int level: Level of details (required)
        :param bool force_refresh: Force to retrieve current available resource information. (required)
        :param str start_date: starting date to check availability from
        :param str end_date: end date to check availability until
        :param str includes: comma separated lists of sites to include
        :param str excludes: comma separated lists of sites to exclude
        :return: Resources
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['level', 'force_refresh', 'start_date', 'end_date', 'includes', 'excludes']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method resources_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'level' is set
        if ('level' not in params or
                params['level'] is None):
            raise ValueError("Missing the required parameter `level` when calling `resources_get`")  # noqa: E501
        # verify the required parameter 'force_refresh' is set
        if ('force_refresh' not in params or
                params['force_refresh'] is None):
            raise ValueError("Missing the required parameter `force_refresh` when calling `resources_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'level' in params:
            query_params.append(('level', params['level']))  # noqa: E501
        if 'force_refresh' in params:
            query_params.append(('force_refresh', params['force_refresh']))  # noqa: E501
        if 'start_date' in params:
            query_params.append(('start_date', params['start_date']))  # noqa: E501
        if 'end_date' in params:
            query_params.append(('end_date', params['end_date']))  # noqa: E501
        if 'includes' in params:
            query_params.append(('includes', params['includes']))  # noqa: E501
        if 'excludes' in params:
            query_params.append(('excludes', params['excludes']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['bearerAuth']  # noqa: E501

        return self.api_client.call_api(
            '/resources', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Resources',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import enum
import traceback
from datetime import datetime
from typing import Tuple, Union, List, Dict

from fim.user import GraphFormat

from fabric_cf.orchestrator import swagger_client
from fim.user.topology import ExperimentTopology, AdvertizedTopology

from fabric_cf.orchestrator.swagger_client import Sliver, Slice, SlicesPost
from fabric_cf.orchestrator.swagger_client.models import PoaPost, PoaData, PoaPostData, Poa


class OrchestratorProxyException(Exception):
    """
    Orchestrator Exceptions
    """
    pass


class SliceState(enum.Enum):
    Nascent = enum.auto()
    Configuring = enum.auto()
    StableError = enum.auto()
    StableOK = enum.auto()
    Closing = enum.auto()
    Dead = enum.auto()
    Modifying = enum.auto()
    ModifyError = enum.auto()
    ModifyOK = enum.auto()
    AllocatedError = enum.auto()
    AllocatedOK = enum.auto()

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name

    @staticmethod
    def state_from_str(state: str):
        if state is None:
            return state

        for t in SliceState:
            if state == str(t):
                return t

        return None

    @staticmethod
    def state_list_to_str_list(states: list):
        if states is None:
            return states

        result = []
        for t in states:
            result.append(str(t))

        return result


@enum.unique
class Status(enum.Enum):
    OK = 1
    INVALID_ARGUMENTS = 2
    FAILURE = 3

    def interpret(self, exception=None):
        interpretations = {
            1: "Success",
            2: "Invalid Arguments",
            3: "Failure"
          }
        if exception is None:
            return interpretations[self.value]
        else:
            return str(exception) + ". " + interpretations[self.value]


class OrchestratorProxy:
    """
    Orchestrator Proxy; must specify the orchestrator host details when instantiating the proxy object
    """
    PROP_AUTHORIZATION = 'Authorization'
    PROP_BEARER = 'Bearer'
    TIME_FORMAT = "%Y-%m-%d %H:%M:%S %z"

    def __init__(self, orchestrator_host: str):
        self.host = orchestrator_host
        self.tokens_api = None
        if orchestrator_host is not None:
            # create_slices an instance of the API class
            configuration = swagger_client.configuration.Configuration()
            #configuration.verify_ssl = False
            configuration.host = f"https://{orchestrator_host}/"
            api_instance = swagger_client.ApiClient(configuration)
            self.slices_api = swagger_client.SlicesApi(api_client=api_instance)
            self.slivers_api = swagger_client.SliversApi(api_client=api_instance)
            self.resources_api = swagger_client.ResourcesApi(api_client=api_instance)
            self.poas_api = swagger_client.PoasApi(api_client=api_instance)
            self.metrics_api = swagger_client.MetricsApi(api_client=api_instance)

    def __set_tokens(self, *, token: str):
        """
        Set tokens
        @param token token
        """
        # Set the tokens
        self.slices_api.api_client.configuration.api_key[self.PROP_AUTHORIZATION] = token
        self.slices_api.api_client.configuration.api_key_prefix[self.PROP_AUTHORIZATION] = self.PROP_BEARER

    def create(self, *, token: str, slice_name: str, ssh_key: Union[str, List[str]],
               topology: ExperimentTopology = None, slice_graph: str = None, lease_start_time: str = None,
               lease_end_time: str = None) -> Tuple[Status, Union[Exception, List[Sliver]]]:
        """
        Create a slice
        @param token fabric token
        @param slice_name slice name
        @param ssh_key SSH Key(s)
        @param topology Experiment topology
        @param slice_graph Slice Graph string
        @param lease_start_time Lease Start Time
        @param lease_end_time Lease End Time
        @return Tuple containing Status and Exception/Json containing slivers created
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token} must be specified")

        if slice_name is None:
            return Status.INVALID_ARGUMENTS, \
                   OrchestratorProxyException(f"Slice Name {slice_name} must be specified")

        if (topology is None and slice_graph is None) or (topology is not None and slice_graph is not None):
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Either topology {topology} or "
                                                                        f"slice graph {slice_graph} must "
                                                                        f"be specified")

        if lease_start_time is not None:
            try:
                datetime.strptime(lease_start_time, self.TIME_FORMAT)
            except Exception as e:
                return Status.INVALID_ARGUMENTS, OrchestratorProxyException(
                    f"Lease Stat Time {lease_start_time} should be in format: {self.TIME_FORMAT} e: {e}")

        if lease_end_time is not None:
            try:
                datetime.strptime(lease_end_time, self.TIME_FORMAT)
            except Exception as e:
                return Status.INVALID_ARGUMENTS, OrchestratorProxyException(
                    f"Lease End Time {lease_end_time} should be in format: {self.TIME_FORMAT} e: {e}")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            if topology is not None:
                slice_graph = topology.serialize()

            if isinstance(ssh_key, str):
                ssh_keys = [ssh_key]
            else:
                ssh_keys = ssh_key
            body = SlicesPost(graph_model=slice_graph, ssh_keys=ssh_keys)
            slivers = self.slices_api.slices_creates_post(name=slice_name, body=body,
                                                          lease_end_time=lease_end_time,
                                                          lease_start_time=lease_start_time)
            return Status.OK, slivers.data if slivers.data is not None else []
        except Exception as e:
            return Status.FAILURE, e

    def modify(self, *, token: str, slice_id: str, topology: ExperimentTopology = None,
               slice_graph: str = None) -> Tuple[Status, Union[Exception, List[Sliver]]]:
        """
        Modify a slice
        @param token fabric token
        @param slice_id slice id
        @param topology Experiment topology
        @param slice_graph Slice Graph string
        @return Tuple containing Status and Exception/Json containing slivers created
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token} must be specified")

        if slice_id is None:
            return Status.INVALID_ARGUMENTS, \
                   OrchestratorProxyException(f"Slice Id {slice_id} must be specified")

        if (topology is None and slice_graph is None) or (topology is not None and slice_graph is not None):
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Either topology {topology} or "
                                                                        f"slice graph {slice_graph} must "
                                                                        f"be specified")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            if topology is not None:
                slice_graph = topology.serialize()

            slivers = self.slices_api.slices_modify_slice_id_put(slice_id=slice_id, body=slice_graph)

            return Status.OK, slivers.data if slivers.data is not None else []
        except Exception as e:
            return Status.FAILURE, e

    def modify_accept(self, *, token: str, slice_id: str) -> Tuple[Status, Union[Exception, ExperimentTopology]]:
        """
        Accept the modify
        @param token fabric token
        @param slice_id slice id
        @return Tuple containing Status and Updated Slice Graph
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token} must be specified")

        if slice_id is None:
            return Status.INVALID_ARGUMENTS, \
                   OrchestratorProxyException(f"Slice Id {slice_id} must be specified")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            slice_details = self.slices_api.slices_modify_slice_id_accept_post(slice_id=slice_id)

            model = slice_details.data[0].model if slice_details.data is not None else None
            topology = None
            if model is not None:
                topology = ExperimentTopology()
                topology.load(graph_string=model)

            return Status.OK, topology
        except Exception as e:
            traceback.print_exc()
            return Status.FAILURE, e

    def delete(self, *, token: str, slice_id: str = None) -> Tuple[Status, Union[Exception, None]]:
        """
        Delete a slice
        @param token fabric token
        @param slice_id slice id
        @return Tuple containing Status and Exception/Json containing deletion status
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token} must be specified")

        try:
            # Set the tokens
            self.slices_api.api_client.configuration.api_key['Authorization'] = token
            self.slices_api.api_client.configuration.api_key_prefix['Authorization'] = 'Bearer'

            if slice_id is not None:
                self.slices_api.slices_delete_slice_id_delete(slice_id=slice_id)
            else:
                self.slices_api.slices_delete_delete()

            return Status.OK, None
        except Exception as e:
            return Status.FAILURE, e

    def slices(self, *, token: str, includes: List[SliceState] = None, excludes: List[SliceState] = None,
               name: str = None, limit: int = 20, offset: int = 0, slice_id: str = None,
               as_self: bool = True, search: str = None, exact_match: bool = False,
               graph_format: str = GraphFormat.GRAPHML.name) -> Tuple[Status, Union[Exception, List[Slice]]]:
        """
        Get slices
        @param token fabric token
        @param includes list of the slice state used to include the slices in the output
        @param excludes list of the slice state used to exclude the slices from the output
        @param name name of the slice
        @param limit maximum number of slices to return
        @param offset offset of the first slice to return
        @param slice_id Slice Id
        @param as_self query as self
        @param search search term
        @param exact_match true if exact match
        @param graph_format graph_format
        @return Tuple containing Status and Exception/Json containing slices
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token} must be specified")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            states = [SliceState.StableError, SliceState.StableOK, SliceState.Nascent,
                      SliceState.Configuring, SliceState.Closing, SliceState.Dead,
                      SliceState.ModifyError, SliceState.ModifyOK, SliceState.Modifying,
                      SliceState.AllocatedOK, SliceState.AllocatedError]
            if includes is not None:
                states = includes

            if excludes is not None:
                for x in excludes:
                    if x in states:
                        states.remove(x)

            if slice_id is not None:
                slices = self.slices_api.slices_slice_id_get(slice_id=slice_id, graph_format=graph_format,
                                                             as_self=as_self)
            elif name is not None:
                slices = self.slices_api.slices_get(states=SliceState.state_list_to_str_list(states), name=name,
                                                    limit=limit, offset=offset, as_self=as_self,
                                                    search=search, exact_match=exact_match)
            else:
                slices = self.slices_api.slices_get(states=SliceState.state_list_to_str_list(states), limit=limit,
                                                    offset=offset, as_self=as_self, search=search,
                                                    exact_match=exact_match)

            return Status.OK, slices.data if slices.data is not None else []
        except Exception as e:
            return Status.FAILURE, e

    def get_slice(self, *, token: str, slice_id: str = None,
                  graph_format: GraphFormat = GraphFormat.GRAPHML,
                  as_self: bool = True) -> Tuple[Status, Union[Exception, ExperimentTopology, dict]]:
        """
        Get slice
        @param token fabric token
        @param slice_id slice id
        @param graph_format
        @param as_self
        @return Tuple containing Status and Exception/Json containing slice
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token} must be specified")

        if slice_id is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Slice Id {slice_id} must be specified")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            slice_details = self.slices_api.slices_slice_id_get(slice_id=slice_id, graph_format=graph_format.name,
                                                                as_self=as_self)

            model = slice_details.data[0].model if slice_details.data is not None else None
            topology = None
            if model is not None:
                topology = ExperimentTopology()
                topology.load(graph_string=model)

            return Status.OK, topology
        except Exception as e:
            return Status.FAILURE, e

    def slivers(self, *, token: str, slice_id: str,
                sliver_id: str = None,
                as_self: bool = True) -> Tuple[Status, Union[Exception, List[Sliver]]]:
        """
        Get slivers
        @param token fabric token
        @param slice_id slice id
        @param sliver_id slice sliver_id
        @param as_self
        @return Tuple containing Status and Exception/Json containing Sliver(s)
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token} must be specified")

        if slice_id is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Slice Id {slice_id} must be specified")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            if sliver_id is None:
                slivers = self.slivers_api.slivers_get(slice_id=slice_id, as_self=as_self)
            else:
                slivers = self.slivers_api.slivers_sliver_id_get(slice_id=slice_id, sliver_id=sliver_id,
                                                                 as_self=as_self)

            return Status.OK, slivers.data if slivers.data is not None else []
        except Exception as e:
            return Status.FAILURE, e

    def resources(self, *, token: str, level: int = 1, force_refresh: bool = False,
                  start: datetime = None, end: datetime = None,
                  includes: List[str] = None, excludes: List[str] = None) -> Tuple[Status, Union[Exception, AdvertizedTopology]]:
        """
        Get resources; by default cached resource information is returned. Cache is refreshed every 5 minutes.
        @param token fabric token
        @param level level
        @param force_refresh force current available resources
        @param start start time
        @param end end time
        @param includes list of sites to include
        @param excludes list of sites to exclude
        @return Tuple containing Status and Exception/Json containing Resources
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token} must be specified")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            start_date = start.strftime(self.TIME_FORMAT) if start else None
            end_date = end.strftime(self.TIME_FORMAT) if end else None
            resources = self.resources_api.resources_get(level=level, force_refresh=force_refresh,
                                                         start_date=start_date, end_date=end_date,
                                                         includes=', '.join(includes) if includes else None,
                                                         excludes=', '.join(excludes) if excludes else None)
            graph_string = resources.data[0].model
            substrate = None
            if graph_string is not None:
                substrate = AdvertizedTopology()
                substrate.load(graph_string=graph_string)

            return Status.OK, substrate
        except Exception as e:
            return Status.FAILURE, e

    def portal_resources(self, *, graph_format: GraphFormat = GraphFormat.JSON_NODELINK) -> Tuple[Status, Union[Exception, AdvertizedTopology]]:
        """
        Get resources for portal
        @param graph_format Graph Format
        @return Tuple containing Status and Exception/Json containing Resources
        """

        try:
            resources = self.resources_api.portalresources_get(graph_format=graph_format.name)

            graph_string = resources.data[0].model
            substrate = None
            if graph_string is not None:
                substrate = AdvertizedTopology()
                substrate.load(graph_string=graph_string)

            return Status.OK, substrate
        except Exception as e:
            return Status.FAILURE, e

    def renew(self, *, token: str, slice_id: str,
              new_lease_end_time: str) -> Tuple[Status, Union[Exception, List, None]]:
        """
        Renew a slice
        @param token fabric token
        @param slice_id slice_id
        @param new_lease_end_time new_lease_end_time
        @return Tuple containing Status and List of Reservation Id failed to extend
        """
        if token is None or slice_id is None or new_lease_end_time is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token}, Slice Id: {slice_id}, "
                                                                        f"New Lease End Time {new_lease_end_time} "
                                                                        f"must be specified")

        try:
            datetime.strptime(new_lease_end_time, self.TIME_FORMAT)
        except Exception as e:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Lease End Time {new_lease_end_time} should "
                                                                        f"be in format: {self.TIME_FORMAT}")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            self.slices_api.slices_renew_slice_id_post(slice_id=slice_id, lease_end_time=new_lease_end_time)

            return Status.OK, None
        except Exception as e:
            return Status.FAILURE, e

    def poa(self, *, token: str, sliver_id: str, operation: str, vcpu_cpu_map: List[Dict[str, str]] = None,
            node_set: List[str] = None, keys: List[Dict[str, str]]) -> Tuple[Status, Union[Exception, List[PoaData]]]:
        """
        Modify a slice
        @param token fabric token
        @param sliver_id Sliver Id
        @param operation POA operation
        @param vcpu_cpu_map vCPU to physical CPU Map
        @param node_set List of Numa nodes
        @param keys list of keys to add/remove
        @return Tuple containing Status and Exception/Json containing poa info created
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token} must be specified")

        if sliver_id is None:
            return Status.INVALID_ARGUMENTS, \
                   OrchestratorProxyException(f"Sliver Id {sliver_id} must be specified")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            body = PoaPost(operation=operation)
            if vcpu_cpu_map is not None or node_set is not None or keys is not None:
                post_data = PoaPostData()
                post_data.vcpu_cpu_map = vcpu_cpu_map
                post_data.node_set = node_set
                post_data.keys = keys
                body.data = post_data

            poa_data = self.poas_api.poas_create_sliver_id_post(sliver_id=sliver_id, body=body)

            return Status.OK, poa_data.data if poa_data.data is not None else None
        except Exception as e:
            return Status.FAILURE, e

    def get_poas(self, *, token: str, sliver_id: str = None,
                 poa_id: str = None, limit: int = 20, offset: int = 0) -> Tuple[Status, Union[Exception, List[PoaData]]]:
        """
        Modify a slice
        @param token fabric token
        @param sliver_id Sliver Id
        @param poa_id POA request id
        @param limit maximum number of poas to return
        @param offset offset of the first poas to return
        @return Tuple containing Status and Exception/Json containing poa info created
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, OrchestratorProxyException(f"Token {token} must be specified")

        if sliver_id is None and poa_id is None:
            return Status.INVALID_ARGUMENTS, \
                   OrchestratorProxyException(f"Sliver Id {sliver_id} or Poa ID {poa_id} must be specified")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            if poa_id is not None:
                poa_data = self.poas_api.poas_poa_id_get(poa_id=poa_id)
            elif sliver_id is not None:
                poa_data = self.poas_api.poas_get(sliver_id=sliver_id, limit=limit, offset=offset)
            else:
                raise Exception("Invalid Arguments")

            return Status.OK, poa_data.data if poa_data.data is not None else None
        except Exception as e:
            return Status.FAILURE, e

    def get_metrics_overview(self, *, token: str = None,
                             excluded_projects: List[str] = None) -> Tuple[Status, Union[Exception, list]]:
        """
        Modify a slice
        @param token fabric token
        @param excluded_projects list of project ids to exclude
        @return Tuple containing Status and Exception/Json containing poa info created
        """
        try:
            # Set the tokens
            self.__set_tokens(token=token)
            result = self.metrics_api.metrics_overview_get(excluded_projects=excluded_projects)
            return Status.OK, result.results
        except Exception as e:
            return Status.FAILURE, e



# coding: utf-8

"""
    Fabric Orchestrator API

    This is Fabric Orchestrator API  # noqa: E501

    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import swagger_client
from swagger_client.api.metrics_api import MetricsApi  # noqa: E501
from swagger_client.rest import ApiException


class TestMetricsApi(unittest.TestCase):
    """MetricsApi unit test stubs"""

    def setUp(self):
        self.api = MetricsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_metrics_overview_get(self):
        """Test case for metrics_overview_get

        Control Framework metrics overview  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Fabric Orchestrator API

    This is Fabric Orchestrator API  # noqa: E501

    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PoaData(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'operation': 'str',
        'poa_id': 'str',
        'state': 'str',
        'sliver_id': 'str',
        'slice_id': 'str',
        'error': 'str',
        'info': 'object'
    }

    attribute_map = {
        'operation': 'operation',
        'poa_id': 'poa_id',
        'state': 'state',
        'sliver_id': 'sliver_id',
        'slice_id': 'slice_id',
        'error': 'error',
        'info': 'info'
    }

    def __init__(self, operation=None, poa_id=None, state=None, sliver_id=None, slice_id=None, error=None, info=None):  # noqa: E501
        """PoaData - a model defined in Swagger"""  # noqa: E501
        self._operation = None
        self._poa_id = None
        self._state = None
        self._sliver_id = None
        self._slice_id = None
        self._error = None
        self._info = None
        self.discriminator = None
        if operation is not None:
            self.operation = operation
        if poa_id is not None:
            self.poa_id = poa_id
        if state is not None:
            self.state = state
        if sliver_id is not None:
            self.sliver_id = sliver_id
        if slice_id is not None:
            self.slice_id = slice_id
        if error is not None:
            self.error = error
        if info is not None:
            self.info = info

    @property
    def operation(self):
        """Gets the operation of this PoaData.  # noqa: E501


        :return: The operation of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._operation

    @operation.setter
    def operation(self, operation):
        """Sets the operation of this PoaData.


        :param operation: The operation of this PoaData.  # noqa: E501
        :type: str
        """

        self._operation = operation

    @property
    def poa_id(self):
        """Gets the poa_id of this PoaData.  # noqa: E501


        :return: The poa_id of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._poa_id

    @poa_id.setter
    def poa_id(self, poa_id):
        """Sets the poa_id of this PoaData.


        :param poa_id: The poa_id of this PoaData.  # noqa: E501
        :type: str
        """

        self._poa_id = poa_id

    @property
    def state(self):
        """Gets the state of this PoaData.  # noqa: E501


        :return: The state of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this PoaData.


        :param state: The state of this PoaData.  # noqa: E501
        :type: str
        """

        self._state = state

    @property
    def sliver_id(self):
        """Gets the sliver_id of this PoaData.  # noqa: E501


        :return: The sliver_id of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._sliver_id

    @sliver_id.setter
    def sliver_id(self, sliver_id):
        """Sets the sliver_id of this PoaData.


        :param sliver_id: The sliver_id of this PoaData.  # noqa: E501
        :type: str
        """

        self._sliver_id = sliver_id

    @property
    def slice_id(self):
        """Gets the slice_id of this PoaData.  # noqa: E501


        :return: The slice_id of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._slice_id

    @slice_id.setter
    def slice_id(self, slice_id):
        """Sets the slice_id of this PoaData.


        :param slice_id: The slice_id of this PoaData.  # noqa: E501
        :type: str
        """

        self._slice_id = slice_id

    @property
    def error(self):
        """Gets the error of this PoaData.  # noqa: E501


        :return: The error of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._error

    @error.setter
    def error(self, error):
        """Sets the error of this PoaData.


        :param error: The error of this PoaData.  # noqa: E501
        :type: str
        """

        self._error = error

    @property
    def info(self):
        """Gets the info of this PoaData.  # noqa: E501


        :return: The info of this PoaData.  # noqa: E501
        :rtype: object
        """
        return self._info

    @info.setter
    def info(self, info):
        """Sets the info of this PoaData.


        :param info: The info of this PoaData.  # noqa: E501
        :type: object
        """

        self._info = info

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PoaData, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PoaData):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
